import argparse
from datastation.ingest_flow import start_import
from datastation.config import init


def main():
    config = init()
    parser = argparse.ArgumentParser(description='Start migration of deposit or batch of deposits')
    parser.add_argument('deposit_path', metavar='<batch-or-deposit>', help='The input file with the dataset pids')
    parser.add_argument('-s', '--single', dest="single_deposit", action="store_true",
                        help="<batch-or-deposit> refers to a single deposit")
    parser.add_argument('-c', '--continue', dest='continue_previous', action='store_true',
                        help="continue previously stopped batch (i.e. allow output directory to be non-empty)")
    parser.add_argument('-d', '--dry-run', dest='dry_run', action='store_true',
                        help='Only print command to be sent to server, but do not actually send it')

    args = parser.parse_args()
    service_baseurl = config['ingest_flow']['service_baseurl']

    start_import(service_baseurl, args.deposit_path, not args.single_deposit, args.continue_previous, is_migration=True,
                 is_dry_run=args.dry_run)


if __name__ == '__main__':
    main()



import argparse
import logging
import os.path
import sys

from datastation.config import init
from datastation.dv_storage import prestage_file, prestage_files, ensure_doi_directory_exists, new_file_json_data_creator


def main():
    config = init()

    parser = argparse.ArgumentParser(
        description='Prestages files in the Dataverse storage directory')
    parser.add_argument('-d', '--doi', dest='doi', help='the dataset DOI', required=True)
    parser.add_argument('file',
                        help='file to prestage; if it is a directory, all files in it will be prestaged recursively')
    args = parser.parse_args()

    file_or_dir = os.path.expanduser(args.file)
    if not os.path.exists(file_or_dir):
        print('File or directory not found: {}'.format(file_or_dir), file=sys.stderr)
        exit(1)

    if args.doi.startswith('doi:'):
        logging.info("Removing 'doi:' prefix from DOI")
        doi = args.doi[4:]
    else:
        doi = args.doi

    files_root = config['dataverse']['files_root']
    ensure_doi_directory_exists(files_root, doi)

    if os.path.isfile(file_or_dir):
        prestage_file(files_root, doi, file_or_dir, new_file_json_data_creator())
    elif os.path.isdir(file_or_dir):
        prestage_files(files_root, doi, file_or_dir, new_file_json_data_creator(file_or_dir))


if __name__ == '__main__':
    main()



import argparse
import logging
import json

from datastation.batch_processing import batch_process
from datastation.config import init
from datastation.ds_pidsfile import load_pids

from datastation.dv_api import get_dataset_locks, delete_dataset_locks


def unlock_dataset_action(server_url, api_token, pid):
    deleted_locks = False
    resp_data = get_dataset_locks(server_url, pid)
    if len(resp_data) == 0:
        logging.debug("{} - No locks found, leave as-is".format(pid))
    else:
        logging.info("{} - Found locks".format(pid))
        logging.debug(json.dumps(resp_data, indent=2))
        logging.debug("{} - Try deleting the locks".format(pid))
        delete_dataset_locks(server_url, api_token, pid)
        deleted_locks = True

    return deleted_locks


def unlock_dataset_command(server_url, api_token, delay, pids_file):
    # could be fast, but depends on number of files inside the dataset
    batch_process(load_pids(pids_file),
                  lambda pid: unlock_dataset_action(server_url, api_token, pid), delay)


def main():
    config = init()
    parser = argparse.ArgumentParser(description='Unlock datasets (if locked) with the pids in the given input file')
    parser.add_argument('-d', '--datasets', dest='pids_file', help='The input file with the dataset pids')
    parser.add_argument('--delay', default=1.5, help="Delay in seconds")
    args = parser.parse_args()

    server_url = config['dataverse']['server_url']
    api_token = config['dataverse']['api_token']

    unlock_dataset_command(server_url, api_token, args.delay, args.pids_file)


if __name__ == '__main__':
    main()



import argparse
from datastation.ingest_flow import start_import
from datastation.config import init


def main():
    config = init()
    parser = argparse.ArgumentParser(description='Print progress report for a batch')
    parser.add_argument('deposits_batch', metavar='<deposits-batch>', help='Path to the batch of deposits to print the progress report for')

    args = parser.parse_args()



if __name__ == '__main__':
    main()



import argparse
import logging
import os

from datastation.batch_processing import batch_process
from datastation.config import init
from datastation.ds_metadatafile import store_dataset_result
from datastation.ds_pidsfile import load_pids
from datastation.dv_api import get_dataset_metadata_export


def retrieve_dataset_metadata_action(server_url, pid, output_dir):
    dataset_metadata = get_dataset_metadata_export(server_url, pid)
    # note that the dataset metadata might be large if there are a lot of files in the dataset!
    store_dataset_result(pid, dataset_metadata, output_dir)


def retrieve_dataset_metadata_command(server_url, delay, input_filename, output_dir):
    logging.info('Args: {}, {}'.format(input_filename, output_dir))
    logging.info("Example using server URL: {}".format(server_url))

    if os.path.isdir(output_dir):
        logging.info("Skipping dir creation, because it already exists: " + output_dir)
    else:
        logging.warning("Creating output dir: " + output_dir)
        os.makedirs(output_dir)

    batch_process(load_pids(input_filename), lambda pid: retrieve_dataset_metadata_action(server_url, pid, output_dir),
                  delay)


def main():
    config = init()
    parser = argparse.ArgumentParser(description='Retrieves the metadata for all published datasets with the pids in '
                                                 'the given inputfile')
    parser.add_argument('-d', '--datasets', dest='pids_file', help='The input file with the dataset pids')
    parser.add_argument('-p', '--pid', help='Pid of single dataset to retrieve the metadata for')
    parser.add_argument('-o', '--output', dest='output_dir',
                        help='The output dir, for storing the metadata files retrieved')
    parser.add_argument('--delay', default=0.2, help="Delay in seconds")
    args = parser.parse_args()

    server_url = config['dataverse']['server_url']
    if args.pid is not None:
        retrieve_dataset_metadata_action(server_url, args.delay, args.pid, args.output_dir)
    else:
        retrieve_dataset_metadata_command(server_url, args.delay, args.pids_file, args.output_dir)


if __name__ == '__main__':
    main()



import argparse
import logging
import json
import sys

from datastation.batch_processing import batch_process
from datastation.config import init
from datastation.ds_pidsfile import load_pids

from datastation.dv_api import replace_dataset_metadatafield, get_dataset_metadata


def replace_metadatafield(server_url, api_token, pid, updated_field):
    logging.debug("{}: Try updating it with: {}".format(pid, updated_field['value']))
    updated_fields = {'fields': [updated_field]}
    logging.debug(json.dumps(updated_fields))
    replace_dataset_metadatafield(server_url, api_token, pid, updated_fields)
    return True


def replace_metadata_field_value(server_url, api_token, pid, field_name, field_from_value, field_to_value):
    # Getting the metadata is not always needed when doing a replacement,
    # but when you need to determine if replace is needed by inspecting the current content
    # you need to 'get' it first.
    # Another approach would be to do that selection up front (via search)
    # and have that generate a list with pids to process 'blindly'.
    resp_data = get_dataset_metadata(server_url, api_token, pid)
    # print(resp_data['datasetPersistentId'])

    replace_field = field_name
    replace_from = field_from_value
    replace_to = field_to_value
    found_replace_from = False
    replaced = False

    for metadata_block in resp_data['metadataBlocks']:
        mdb_fields = resp_data['metadataBlocks'][metadata_block]['fields']

        for field in mdb_fields:
            # expecting (assuming) one and only one instance,
            # but the code will try to change all it can find
            if field['typeName'] == replace_field:
                logging.debug("{}: Found {} with value {} ".format(pid, field['typeName'],  field['value']))
                if not field['typeClass'] == 'primitive':
                    sys.exit("field {} does not have typeClass=`primitive` but {}".format(replace_field, field['typeClass']))
                # be safe and mutate a copy
                updated_field = field.copy()
                if field['multiple'] == 'true':
                    try:
                        index_replace_from = field['value'].index(replace_from)
                        found_replace_from = True
                        updated_field['value'][index_replace_from] = replace_to
                        replaced = replace_metadatafield(server_url, api_token, pid, updated_field)
                        logging.debug("{}: Updated {} from {} to {}".format(pid, replace_field, replace_from, replace_to))
                    except ValueError:
                        logging.debug("{} not found in {}".format(replace_from, field['value']))
                else:
                    if field['value'] == replace_from:
                        found_replace_from = True
                        updated_field['value'] = replace_to
                        replaced = replace_metadatafield(server_url, api_token, pid, updated_field)
                        logging.debug("{}: Updated {} from {} to {}".format(pid, replace_field, replace_from, replace_to))
                    else:
                        logging.debug("Found {} instead of {}, Leave as-is".format(field['value'], replace_from))
    if not found_replace_from:
        logging.debug("{}: {} not found, nothing to replace".format(pid, replace_field))
    return replaced


def replace_metadata_field_value_command(server_url, api_token, delay, pids_file, field_name,
                                         field_from_value, field_to_value):
    pids = load_pids(pids_file)

    batch_process(pids,
                  lambda pid: replace_metadata_field_value(server_url, api_token, pid, field_name,
                                                                  field_from_value,  field_to_value), delay)


# Note that the datasets that got changed get into a DRAFT status
# and at some point need to be published with a minor version increment.
# This is not done here, because you might want several (other) changes
# on the same datasets before publishing.
def main():
    config = init()
    parser = argparse.ArgumentParser(
        description='Replace metadata field in datasets with the dois in the given input file. See the json metadata '
                    'export (dataverse_json) to see what names are possible for the fields and metadata blocks. The '
                    'field must have typeClass=`primitive`. The field must already be present.')

    parser.add_argument("-n", "--field-name", help="Name of the primitive field (json typeName)", dest="field_name")
    parser.add_argument("-f", "--from-value", help="Value to be replaced", dest="field_from_value")
    parser.add_argument("-t", "--to-value", help="The replacement value (the new value)", dest="field_to_value")
    parser.add_argument('-d', '--datasets', dest='pids_file', help='The input file with the dataset dois')
    parser.add_argument('-p', '--pid', help="Doi of the dataset for which to replace the metadata.")
    parser.add_argument('--delay', default=5.0, help="Delay in seconds")
    args = parser.parse_args()

    server_url = config['dataverse']['server_url']
    api_token = config['dataverse']['api_token']

    if args.pid is not None:
        replace_metadata_field_value(server_url, api_token, args.pid, args.field_name,
                                      args.field_from_value, args.field_to_value)
    else:
        replace_metadata_field_value_command(server_url, api_token, args.delay, args.pids_file,
                                             args.field_name, args.field_from_value, args.field_to_value)


if __name__ == '__main__':
    main()



import argparse
import logging
import os.path
import shutil

from datastation.config import init
from datastation.ingest_flow import set_permissions, is_subpath_of, is_dir_in_inbox


def main():
    config = init()
    parser = argparse.ArgumentParser(description='Move a batch to an ingest area')
    parser.add_argument('source', metavar='<source>', help='Source batch to move')
    parser.add_argument('dest', metavar='<dest>', help='Destination to move to. If the destination does not exist it is'
                                                       ' created and the contents of the batch is moved to it.')
    args = parser.parse_args()

    src = args.source
    batch_name = os.path.basename(src)
    dest = args.dest

    if os.path.isdir(dest):
        dest = os.path.join(dest, batch_name)

    dir_mode = int(config['ingest_flow']['deposits_mode']['directory'], 8)
    file_mode = int(config['ingest_flow']['deposits_mode']['file'], 8)
    deposits_group = config['ingest_flow']['deposits_group']
    ingest_areas = config['ingest_flow']['ingest_areas']
    inboxes = list(map(lambda a: a['inbox'], ingest_areas.values()))
    if is_dir_in_inbox(dest, inboxes):
        logging.debug("Moving from {} to {}".format(args.source, dest))
        shutil.move(src=args.source, dst=dest)
        logging.debug("Setting permissions and group...")
        set_permissions(dest, dir_mode, file_mode, deposits_group)
        logging.info("Done")
    else:
        print("Destination {} is not located in a configured ingest area inbox (one of {})".format(dest,
                                                                                                   ", ".join(inboxes)))
        exit(1)



import argparse
from datastation.ingest_flow import start_import, list_events
from datastation.config import init


def main():
    config = init()
    parser = argparse.ArgumentParser(description='List the events per deposit')
    parser.add_argument('-s', '--source',
                        dest='source',
                        metavar='<source>',
                        help='only events for this source (i.e. batch or auto-ingest)')
    parser.add_argument('-d', '--deposit',
                        metavar='<uuid>',
                        dest='deposit_id',
                        help='only events for this deposit')
    args = parser.parse_args()

    params = {}
    if args.source is not None:
        params['source'] = args.source
    if args.deposit_id is not None:
        params['depositId'] = args.deposit_id
    list_events(params)


if __name__ == '__main__':
    main()



import argparse
import datetime
import logging

from datastation.batch_processing import batch_process
from datastation.config import init
from datastation.ds_pidsfile import load_pids
from datastation.dv_api import add_dataset_role_assignment, get_dataset_roleassigments
from datastation.open_csv_file import open_csv_file


def add_roleassignment(server_url, api_token, csv_writer, pid, role_assignee, role_alias, dry_run):
    current_assignments = get_dataset_roleassigments(server_url, api_token, pid)
    found = False
    for current_assignment in current_assignments:
        if current_assignment.get('assignee') == role_assignee and current_assignment.get('_roleAlias') == role_alias:
            found = True
            break

    action = "None" if found else "Added"

    csv_writer.writerow({'DOI': pid, 'Modified': datetime.datetime.now(), 'Assignee': role_assignee, 'Role': role_alias,
                         'Change': action})

    if found:
        logging.warning("{} is already {} for dataset {}".format(role_assignee, role_alias, pid))
    else:
        if dry_run:
            logging.info("Dry run: not adding {} as {} for dataset {}".format(role_assignee, role_alias, pid))
        else:
            role_assignment = {"assignee": role_assignee, "role": role_alias}
            add_dataset_role_assignment(server_url, api_token, pid, role_assignment)


def add_roleassignment_command(server_url, api_token, output_file, pids_file, role_assignee, role_alias, dry_run):
    pids = load_pids(pids_file)

    headers = ["DOI", "Modified", "Assignee", "Role", "Change"]
    csv_file, csv_writer = open_csv_file(headers, output_file)

    batch_process(pids, lambda pid: add_roleassignment(server_url, api_token, csv_writer, pid, role_assignee,
                                                       role_alias, dry_run))

    if not output_file == '-':
        csv_file.close()


def main():
    config = init()
    parser = argparse.ArgumentParser(description='Add role assignment to specified datasets')
    parser.add_argument('-d', '--datasets', dest='pid_file', help='The input file with the dataset pids')
    parser.add_argument('--dry-run', dest='dry_run', help="only logs the actions, nothing is executed", action='store_true')
    parser.add_argument('-a', "--role-assignment", help="Role assignee and alias (example: @dataverseAdmin=contributor)")
    parser.add_argument('-r', '--report', required=True, dest='output_file',
                        help="Destination of the output report file, '-' sends it to stdout")
    args = parser.parse_args()

    role_assignee = args.role_assignment.split('=')[0]
    role_alias = args.role_assignment.split('=')[1]

    server_url = config['dataverse']['server_url']
    api_token = config['dataverse']['api_token']
    add_roleassignment_command(server_url, api_token, args.output_file, args.pid_file, role_assignee, role_alias,
                               args.dry_run)


if __name__ == '__main__':
    main()



import argparse
import logging
import os

from datetime import datetime
from lxml import etree

from datastation.config import init
from datastation.dv_api import get_oai_records, get_oai_records_resume


def save_oai_records(xml_doc, counter, records_output_dir):
    # Improve the next
    # count number of records and print it
    # print(len(xml_doc.findall('.//{http://www.openarchives.org/OAI/2.0/}record')))

    xml_filename = 'recordset_' + str(counter) + '.xml'
    f = open(os.path.join(records_output_dir, xml_filename), "wb")
    # Note that we don't have the XML declaration
    f.write(etree.tostring(xml_doc, pretty_print=True))
    f.close()
    # Note: maybe extract the dataset specific parts and save as individual xml files?


def oai_harvest_command(server_url, output_dir, format, set=None):
    # create directory for this export
    timestamp_str = datetime.now().strftime("%Y%m%d_%H%M%S")
    harvest_dirname = 'oai_harvest_' + format + '_' + timestamp_str
    records_output_dir = os.path.join(output_dir, harvest_dirname)
    os.makedirs(records_output_dir)

    logging.info("Harvesting {} from {} in format {}".format(set, server_url, format))
    logging.info("Storing results in {}".format(os.path.abspath(records_output_dir)))
    xml_doc = get_oai_records(server_url, format=format, set=set)

    counter = 0
    save_oai_records(xml_doc, counter, records_output_dir)

    # get the resumptionToken, sax would be more efficient than DOM here!
    # OAI-PMH/ListRecords/resumptionToken
    token = xml_doc.find('.//{http://www.openarchives.org/OAI/2.0/}resumptionToken')

    # The resumptionToken is empty (no text) when we have all the records
    while token is not None and token.text is not None:
        logging.info("In page {} of the recordset, resumption token found: {}".format(counter, token.text))
        xml_doc = get_oai_records_resume(server_url, token.text)
        counter += 1
        save_oai_records(xml_doc, counter, records_output_dir)
        token = xml_doc.find('.//{http://www.openarchives.org/OAI/2.0/}resumptionToken')


def main():
    config = init()
    parser = argparse.ArgumentParser(description='Harvest dataset metadata records via the OAI-PMH protocol')
    parser.add_argument('-f', '--format', default='oai_dc',
                        help='The PMH metadataPrefix in which the metadata should be harvested.')
    parser.add_argument('-s', '--set', default='', help='The PMH recordset to be harvested.')
    parser.add_argument('-o', '--output-dir', dest='output_dir', required=True,
                        help="The folder to store the exported metadata records")
    args = parser.parse_args()

    oai_format = args.format  # Note that an important one we have in dataverse is 'oai_datacite'
    oai_set = None
    if args.set:
        oai_set = args.set

    server_url = config['dataverse']['server_url']

    oai_harvest_command(server_url, args.output_dir, format=oai_format, set=oai_set)


if __name__ == '__main__':
    main()



import argparse

from datastation.batch_processing import batch_process
from datastation.config import init
from datastation.ds_pidsfile import load_pids
from datastation.dv_api import reindex_dataset


def reindex_dataset_command(server_url, delay, pids_file):
    pids = load_pids(pids_file)

    # could be fast, but depends on number of files inside the dataset
    batch_process(pids, lambda pid: reindex_dataset(server_url, pid), delay)


def main():
    config = init()
    parser = argparse.ArgumentParser(description='Reindex datasets with the pids in the given input file')
    parser.add_argument('-d', '--datasets', dest='pids_file', help='The input file with the dataset dois with '
                                                                     'pattern doi:prefix/shoulder/postfix')
    parser.add_argument('--delay', default=2.0, help="Delay in seconds")
    args = parser.parse_args()

    server_url = config['dataverse']['server_url']

    reindex_dataset_command(server_url, args.delay, args.pids_file)


if __name__ == '__main__':
    main()



import argparse
import csv
import json
import logging
import psycopg
import requests
import sys

from datastation.config import init


def import_user(dv_server_url, add_builtin_users_key, user, dvndb_conn, dryrun):
    logging.info("Add user {} with email {}".format(user["userName"], user["email"]))
    dummy_password = "1234AB"

    user_json = json.dumps(user)
    api_call = "{}/api/builtin-users?password={}&key={}&sendEmailNotification=false".format(
        dv_server_url, dummy_password, add_builtin_users_key)
    if dryrun:
        logging.info("dry-run, not calling {}".format(api_call))
        logging.debug(user_json)
    else:
        header = {'Content-Type': 'application/json'}
        dv_resp = requests.post(api_call, data=user_json, headers=header)
        response = dv_resp.json()
        if response["status"] == "ERROR":
            logging.error("response from dataverse API: {}".format(response["message"]))
            logging.info("NOT updating password for user {} in database".format(user["userName"]))
            return

    update_statement = "UPDATE builtinuser SET encryptedpassword = '{}' , passwordencryptionversion = 0 " \
                       "WHERE username = '{}'".format(user["encrypted_password"], user["userName"])
    if dryrun:
        logging.info("dry-run, not updating database with {}".format(update_statement))
    else:
        with dvndb_conn.cursor() as dvndb_cursor:
            try:
                dvndb_cursor.execute(update_statement)
                dvndb_conn.commit()
            except psycopg.DatabaseError as error:
                logging.error(error)
                sys.exit("FATAL ERROR: problem updating dvndb password for user {}".format(user["userName"]))


def connect_to_database(host: str, db: str, user: str, password: str):
    return psycopg.connect(
        "host={} dbname={} user={} password={}".format(host, db, user, password))


def main():
    config = init()

    parser = argparse.ArgumentParser(description='Import users into a Dataverse using the BuiltinUsers.KEY')
    parser.add_argument('--easy', dest='is_easy_format',
                        help="The csv file is exported from EASY and has the following columns: UID, INITIALS, SURNAME,"
                             "PREFIX, EMAIL, ORGANISATION, FUNCTION, PASSWORD-HASH. "
                             "If not set, the following columns are expected: Username, GivenName, FamilyName, Email, "
                             "Affiliation, Position, encryptedpassword",
                        action='store_true')
    parser.add_argument('-k', '--builtin-users-key', help="BuiltinUsers.KEY set in Dataverse")
    parser.add_argument('-r', '--dry-run', dest='dry_run', help="only logs the actions, nothing is executed",
                        action='store_true')
    parser.add_argument('-i', '--input-csv', help="the csv file containing the users and hashed passwords")
    args = parser.parse_args()

    if (not args.dry_run) and args.builtin_users_key is None:
        sys.exit("Exiting: Not in dry-run mode but no builtin_users_key provided")

    dvndb_conn = None
    try:
        if not args.dry_run:
            dvndb_conn = connect_to_database(config['dataverse']['db']['host'], config['dataverse']['db']['dbname'],
                                             config['dataverse']['db']['user'], config['dataverse']['db']['password'])

        with open(args.input_csv, "r") as input_file_handler:
            csv_delimeter = ';' if args.is_easy_format else ','
            csv_reader = csv.DictReader(input_file_handler, delimiter=csv_delimeter)

            for row in csv_reader:
                if args.is_easy_format:
                    last_name = (row["PREFIX"], row["SURNAME"])
                    # the PASSWORD-HASH starts with '{SHA}' which needs to be stripped
                    encrypted_password = row["PASSWORD-HASH"][5:]
                    user = {"userName": row["UID"], "firstName": row["INITIALS"], "lastName": " ".join(last_name),
                            "email": row["EMAIL"], "affiliation": row["ORGANISATION"], "position": row["FUNCTION"],
                            "encrypted_password": encrypted_password}
                else:
                    user = {"userName": row["Username"], "firstName": row["GivenName"],
                            "lastName": row["FamilyName"], "email": row["Email"], "affiliation": row["Affiliation"],
                            "position": row["Position"], "encrypted_password": row["encryptedpassword"]}

                import_user(config['dataverse']['server_url'], args.builtin_users_key, user, dvndb_conn, args.dry_run)

        if not args.dry_run:
            dvndb_conn.close()
    except psycopg.DatabaseError as error:
        logging.error(error)
    finally:
        if dvndb_conn is not None:
            dvndb_conn.close()


if __name__ == '__main__':
    main()



import argparse
import json
import logging

from datastation.config import init
from datastation.dv_api import get_dataset_metadata


def retrieve_dataset_metadata_field(server_url, api_token, pid, mdb_name, field_name):
    resp_data = get_dataset_metadata(server_url, api_token, pid)
    mdb_fields = resp_data['metadataBlocks'][mdb_name]['fields']
    found = False
    for field in mdb_fields:
        if field['typeName'] == field_name:
            print(json.dumps(field))
            found = True
    if not found:
        logging.info("No field {} found in {} for dataset {}".format(field_name, mdb_name, pid))


def main():
    config = init()
    parser = argparse.ArgumentParser(description='Retrieves the json for the given field for a dataset')
    parser.add_argument('-p', '--pid', help='The pid of the dataset', required=True)
    parser.add_argument('-m', '--metadata_block',
                        help='The metadata_block that holds the target field')
    parser.add_argument('-f', '--field', required=True, help='The target field')
    args = parser.parse_args()

    server_url = config['dataverse']['server_url']
    api_token = config['dataverse']['api_token']
    retrieve_dataset_metadata_field(server_url, api_token, args.pid, args.metadata_block, args.field)


if __name__ == '__main__':
    main()


import argparse
import json
import logging
import os, grp
import shutil

import requests
from builtins import all
import stat

file_writeable_to_group = lambda f: os.stat(f).st_mode & stat.S_IWGRP > 0

def start_import(service_baseurl, path, is_batch, continue_previous, is_migration, is_dry_run):
    if not has_dirtree_pred(path, file_writeable_to_group):
        chmod_command = "chmod -R g+w %s" % path
        print("Some files in the import batch do not give the owner group write permissions. Executing '%s' to fix it" % chmod_command)
        status = os.system(chmod_command)
        if status != 0:
            print("Could not give owner group write permissions. Possibly your account is not the owner user of the files.")
            return
    print("Sending start import request to server...")
    command = {
        'inputPath': os.path.abspath(path),
        'batch': is_batch,
        'continue': continue_previous
    }
    if is_dry_run:
        logging.info("Only printing command, not sending it...")
        print(json.dumps(command, indent=2))
    else:
        r = requests.post('%s/%s/:start' % (service_baseurl, "migrations" if is_migration else "imports"), json=command)
        print('Server responded: %s' % r.text)

def has_file_pred(file, pred):
    return pred(file)

def has_dirtree_pred(dir, pred):
    for root, dirs, files in os.walk(dir):
        return pred(root) \
               and all(pred(os.path.join(root, dir)) for dir in dirs) \
               and all(pred(os.path.join(root, file)) for file in files)

def list_events(service_baseurl, params):
    r = requests.get('%s/events' % service_baseurl, headers={'Accept': 'text/csv'}, params=params)
    print(r.text)

def set_permissions(dir, dir_mode, file_mode, group):
    for root, dirs, files in os.walk(dir):
        for d in [root] + dirs:
            p = os.path.join(root, d)
            os.chmod(p, dir_mode)
            shutil.chown(p, group=group)
        for f in files:
            p = os.path.join(root, f)
            os.chmod(p, file_mode)
            shutil.chown(p, group=group)

def is_dir_in_inbox(dir, inboxes):
    return next(filter(lambda p: is_subpath_of(dir, p),  inboxes), None) is not None

def is_subpath_of(dir, parent):
    absolute_parent = os.path.abspath(parent)
    absolute_dir = os.path.abspath(dir)
    return absolute_dir.startswith(absolute_parent)

def progress_report(deposit_dir, ingest_areas):
    # find the ingest area
    # find the relative path (assume first dir is inbox?)
    #
    return ""



import json
import logging
import os.path
from uuid import uuid4
from datetime import datetime
from os import walk, path
from shutil import copy
import hashlib


# Taken from edu.harvard.iq.dataverse.DataFileServiceBean#generateStorageIdentifier()
def generate_storage_identifier():
    uuid = str(uuid4())
    hexRandom = uuid[-12:]
    hexTimestamp = hex(int(datetime.now().timestamp() * 1000))[2:]  # remove 0x from beginning
    return "{}-{}".format(hexTimestamp, hexRandom)


def calc_sha1(file):
    sha1sum = hashlib.sha1()
    with open(file, 'rb') as source:
        block = source.read(2 ** 16)
        while len(block) != 0:
            sha1sum.update(block)
            block = source.read(2 ** 16)
    return sha1sum.hexdigest()


def new_file_json_data_creator(inputroot=None):
    logging.info("Creating directory names relative to: {}".format(inputroot))
    def create_json_data_for_file(inputfile, storage_identifier):
        if inputroot is not None:
            dir_label = os.path.dirname(os.path.relpath(inputfile, inputroot))
        else:
            dir_label = ""

        d = {
            "directoryLabel": dir_label,
            "fileName": os.path.basename(inputfile),
            "description": "",
            "categories": [],
            "restrict": "false",
            "storageIdentifier": "file://{}".format(storage_identifier),
            "mimeType": "application/octet-stream",
            "checksum": {
                "@type": "SHA-1",
                "@value": calc_sha1(inputfile)
            }
        }
        print(json.dumps(d))

    return create_json_data_for_file


def prestage_file(fileroot, doi, f, create_json_data):
    storage_id = generate_storage_identifier()
    dest = path.join(fileroot, doi, storage_id)
    logging.debug("Copying {} to {}".format(f, dest))
    copy(f, dest)
    create_json_data(f, storage_id)


def prestage_files(fileroot, doi, dir, create_json_data):
    print("[")
    first = True
    for root, _, files in walk(dir):
        for f in files:
            if (not first):
                print(", ")
            prestage_file(fileroot, doi, os.path.join(root, f), create_json_data)
            first = False
    print("]")

def ensure_doi_directory_exists(fileroot, doi):
    doi_dir = path.join(fileroot, doi)
    if os.path.isdir(doi_dir):
        logging.debug("DOI directory already exists: {}".format(doi_dir))
    else:
        logging.info("Creating new DOI directory: {}".format(doi_dir))
        os.makedirs(doi_dir)



import sys
import argparse
import logging

from datastation.batch_processing import batch_process
from datastation.config import init
from datastation.ds_pidsfile import load_pids
from datastation.dv_api import delete_dataset_draft


def delete_dataset_command(server_url, api_token, pids_file, delay):
    pids = load_pids(pids_file)

    batch_process(pids, lambda pid: delete_dataset_draft(server_url, api_token, pid), delay)


def main():
    config = init()
    parser = argparse.ArgumentParser(description='Delete datasets with the pids in the given input file. '
                                                 'Only the draft version is deleted '
                                                 'and it will fail if it is not a draft!')
    parser.add_argument('-d', '--datasets', dest='dataset_pids_file', required=True,
                        help='The input file with the dataset pids with pattern doi:prefix/shoulder/postfix')
    parser.add_argument('--delay', default=2.0, help="Delay in seconds")
    args = parser.parse_args()

    server_url = config['dataverse']['server_url']
    api_token = config['dataverse']['api_token']

    logging.info("Deleting draft datasets on: {}".format(server_url))
    logging.warning("Be aware that this is irreversible and you might lose information!")
    # Only proceed if user is sure
    if not input("Are you sure? (y/n): ").lower().strip()[:1] == "y":
        logging.info("Cancelling deletion")
        sys.exit(1)
    logging.info("Starting deletion")

    delete_dataset_command(server_url, api_token, pids_file=args.dataset_pids_file, delay=args.delay)


if __name__ == '__main__':
    main()



import json

import requests

from lxml import etree


# Thin 'client' functions for http API on the dataverse service, not using a API lib, just the requests lib
# could be placed in a class that also keeps hold of the url and token that we initialise once!
#
# Also note that here we use the PID (persistentId) instead of the internal ID form of the requests.


def search(server_url, subtree, start=0, rows=10):
    '''
    Do a query via the public search API, only published datasets
    using the public search 'API', so no token needed

    Note that the current functionality of this function is very limited!

    :param subtree: This is the collection (dataverse alias)
                    it recurses into a collection and its children etc. very useful with nesting collection
    :param start: The cursor (zero based result index) indicating where the result page starts
    :param rows: The number of results returned in the 'page'
    :return: The 'paged' search results in a list of dictionaries
    '''

    # always type=dataset, those have pids (disregarding pids for files)
    params = {
        'q': '*',
        'subtree': subtree,
        'type': 'dataset',
        'per_page': str(rows),
        'start': str(start)
    }

    # params['fq'] = ''

    dv_resp = requests.get(server_url + '/api/search', params=params)

    # give some feedback
    # print("Status code: {}".format(dv_resp.status_code))
    # print("Json: {}".format(dv_resp.json()))
    # the json result is a dictionary... so we could check for something in it
    dv_resp.raise_for_status()
    resp_data = dv_resp.json()['data']
    # print(json.dumps(resp_data, indent=2))
    return resp_data


def get_dataset_metadata_export(server_url, pid, exporter = 'dataverse_json', response_is_json = True):
    params = {'exporter': exporter, 'persistentId': pid}
    dv_resp = requests.get(server_url + '/api/datasets/export',
                           params=params)

    # give some feedback
    # print("Status code: {}".format(dv_resp.status_code))
    # print("Json: {}".format(dv_resp.json()))
    # the json result is a dictionary... so we could check for something in it
    dv_resp.raise_for_status()
    # assume json, but not all exporters have that!
    if response_is_json:
        resp_data = dv_resp.json()  # Note that the response json has no wrapper around the data
    else:
        resp_data = dv_resp.text
    return resp_data


# with a token, can also get metadata from drafts
def get_dataset_metadata(server_url, api_token, pid):
    headers = {'X-Dataverse-key': api_token}
    dv_resp = requests.get(server_url + '/api/datasets/:persistentId/versions/:latest?persistentId=' + pid,
                           headers=headers)
    # Maybe give some more feedback
    # print("Status code: {}".format(dv_resp.status_code))
    # print("Json: {}".format(dv_resp.json()))
    # the json result is a dictionary... so we could check for something in it
    dv_resp.raise_for_status()
    resp_data = dv_resp.json()['data']
    return resp_data


# note that the dataset will become a draft if it was not already
def replace_dataset_metadatafield(server_url, api_token, pid, field):
    headers = {'X-Dataverse-key': api_token}
    try:
        dv_resp = requests.put(
            server_url + '/api/datasets/:persistentId/editMetadata?persistentId=' + pid + '&replace=true',
            data=json.dumps(field, ensure_ascii=False),
            headers=headers)
        dv_resp.raise_for_status()
    except requests.exceptions.RequestException as re:
        print("RequestException: ", re)
        raise
    resp_data = dv_resp.json()['data']
    return resp_data

def get_dataset_roleassigments(server_url, api_token, pid):
    headers = {'X-Dataverse-key': api_token}
    params = {'persistentId': pid}
    try:
        dv_resp = requests.get(server_url + '/api/datasets/:persistentId/assignments',
                               params=params,
                               headers=headers)
        dv_resp.raise_for_status()
    except requests.exceptions.RequestException as re:
        print("RequestException: ", re)
        raise
    resp_data = dv_resp.json()['data']
    return resp_data


def delete_dataset_role_assignment(server_url, api_token, pid, assignment_id):
    headers = {'X-Dataverse-key': api_token}
    dv_resp = requests.delete(server_url + '/api/datasets/:persistentId/assignments/' + str(assignment_id)
                              + '?persistentId=' + pid,
                              headers=headers)
    dv_resp.raise_for_status()


def add_dataset_role_assignment(server_url, api_token, pid, assignment):
    headers = {'X-Dataverse-key': api_token, 'Content-Type': 'application/json'}
    params = {'persistentId': pid}
    dv_resp = requests.post(server_url + '/api/datasets/:persistentId/assignments/',
                            headers=headers,
                            data=json.dumps(assignment, ensure_ascii=False),
                            params=params)
    dv_resp.raise_for_status()


def get_dataset_locks(server_url, pid):
    dv_resp = requests.get(server_url + '/api/datasets/:persistentId/locks?persistentId=' + pid)
    # give some feedback
    # print("Status code: {}".format(dv_resp.status_code))
    # print("Json: {}".format(dv_resp.json()))
    # the json result is a dictionary... so we could check for something in it
    dv_resp.raise_for_status()
    resp_data = dv_resp.json()['data']
    return resp_data


def delete_dataset_locks(server_url, api_token, pid):
    headers = {'X-Dataverse-key': api_token}
    dv_resp = requests.delete(server_url + '/api/datasets/:persistentId/locks?persistentId=' + pid,
                              headers=headers)
    dv_resp.raise_for_status()


def publish_dataset(server_url, api_token, pid, version_upgrade_type="major"):
    # version_upgrade_type must be 'major' or 'minor', indicating which part of next version to increase
    headers = {'X-Dataverse-key': api_token}
    dv_resp = requests.post(server_url + '/api/datasets/:persistentId/actions/:publish?persistentId='
                            + pid + '&type=' + version_upgrade_type,
                            headers=headers)
    dv_resp.raise_for_status()


# This is via the admin api and does not use the token,
# but instead will need to be run on localhost or via an SSH tunnel for instance!
def reindex_dataset(server_url, pid):
    dv_resp = requests.get(server_url + '/api/admin/index/dataset?persistentId=' + pid)
    dv_resp.raise_for_status()
    resp_data = dv_resp.json()['data']
    return resp_data


# Warning: this deletes information that might be difficult to restore
# Use mostly while developing and testing with non-production data.
def delete_dataset_draft(server_url, api_token, pid):
    headers = {'X-Dataverse-key': api_token}
    dv_resp = requests.delete(server_url + '/api/datasets/:persistentId/versions/:draft?persistentId=' + pid,
                              headers=headers)
    dv_resp.raise_for_status()


# Warning: this deletes also a PUBLISHED dataset!
# Use mostly while developing and testing with non-production data.
# The reasons to destroy on production should be extremely rare!
def destroy_dataset(server_url, api_token, pid):
    headers = {'X-Dataverse-key': api_token}
    dv_resp = requests.delete(server_url + '/api/datasets/:persistentId/destroy/?persistentId=' + pid,
                              headers=headers)
    dv_resp.raise_for_status()


# Remember to get info on the OAI endpoint you can do:
# oai?verb=Identify
# oai?verb=ListSets
# oai?verb=ListMetadataFormats
# we could add function for that if we wanted
#
# Default there is no set specified and you get just all
# also no date range (with from, until)
def get_oai_records(server_url, format, set=None):
    params = {'verb': 'ListRecords', 'metadataPrefix': format}
    if set is not None:
        params['set'] = set
    dv_resp = requests.get(server_url + '/oai',
                           params=params)

    dv_resp.raise_for_status()
    # assume XML
    xml_doc = etree.fromstring(dv_resp.content)
    # alternatively we could use the parse directly and not requests.get
    # xml_doc = etree.parse(url).getroot()

    return xml_doc


def get_oai_records_resume(server_url, token):
    params = {'verb': 'ListRecords', 'resumptionToken': token}
    dv_resp = requests.get(server_url + '/oai',
                           params=params)

    dv_resp.raise_for_status()
    # assume XML
    xml_doc = etree.fromstring(dv_resp.content)
    return xml_doc



import argparse
import logging
import sys
import psycopg

from datastation.config import init


def find_datasets_with_roleassignment(dvndb_conn, role_assignee, role_alias):
    select_statement = \
        "select concat(dvo.protocol, ':', dvo.authority, '/', dvo.identifier) " \
        "from roleassignment ra inner join dataverserole dr on ra.role_id=dr.id " \
        "inner join dvobject dvo on definitionpoint_id=dvo.id " \
        "where dtype='Dataset' and assigneeidentifier='{}' and alias='{}'".format(role_assignee, role_alias)

    with dvndb_conn.cursor() as dvndb_cursor:
        try:
            dvndb_cursor.execute(select_statement)
            result = dvndb_cursor.fetchall()
            if len(result) == 0:
                logging.info("No datasets for user {} with role {}".format(role_assignee, role_alias))
            for r in result:
                print(r[0])
        except psycopg.DatabaseError as error:
            logging.error(error)
            sys.exit("FATAL ERROR: problem accessing database with {} ".format(select_statement))


def main():
    config = init()
    parser = argparse.ArgumentParser(description='Find datasets that are linked to a specific account')
    parser.add_argument("role_assignment", help="Role assignee and alias (example: @dataverseAdmin=contributor)")
    args = parser.parse_args()

    role_assignee = args.role_assignment.split('=')[0]
    role_alias = args.role_assignment.split('=')[1]

    dvndb_conn = None
    try:
        dvndb_conn = connect_to_database(config['dataverse']['db']['host'], config['dataverse']['db']['dbname'],
                                         config['dataverse']['db']['user'], config['dataverse']['db']['password'])
        find_datasets_with_roleassignment(dvndb_conn, role_assignee, role_alias)
    except psycopg.DatabaseError as error:
        logging.error(error)
        sys.exit("FATAL ERROR: No connection with database established: {}".format(error))
    finally:
        if dvndb_conn is not None:
            dvndb_conn.close()


def connect_to_database(host, dbname, user: str, password: str):
    return psycopg.connect(
        "host={} dbname={} user={} password={}".format(host, dbname, user, password))


if __name__ == '__main__':
    main()



import csv
import sys


def open_csv_file(headers, output_file):
    csv_file = None
    if output_file == '-':
        csv_writer = csv.DictWriter(sys.stdout, headers)
    else:
        csv_file = open(output_file, 'w')
        csv_writer = csv.DictWriter(csv_file, headers)

    csv_writer.writeheader()
    return csv_file, csv_writer



import argparse
import logging

from datastation.config import init
from datastation.ds_pidsfile import store_pids
from datastation.dv_search import get_dataset_pids_from_search


def retrieve_dataset_pids_command(server_url, dataverse_alias, output_dir, output_filename):
    logging.info("Retrieving dataset PIDs from URL {}, dataverse {}".format(server_url, dataverse_alias))
    pids = get_dataset_pids_from_search(server_url, dataverse_alias)
    store_pids(pids, output_dir, output_filename)


def main():
    config = init()

    parser = argparse.ArgumentParser(
        description='Retrieves the pids for all published datasets in the given collection')
    parser.add_argument('-o', '--output-dir', dest='output_dir', required=True,
                        help='The output file, for storing the pids retrieved')
    parser.add_argument('-f', '--output-file', dest='output_file',
                        help='name of the output file that will contain all the pids of the published datasets. '
                             'Defaults to pids-<timestamp>.txt')
    parser.add_argument('dataverse_alias', help='The short name (or alias) of the dataverse (collection)')
    args = parser.parse_args()

    retrieve_dataset_pids_command(config['dataverse']['server_url'], args.dataverse_alias, args.output_dir,
                                  args.output_file)


if __name__ == '__main__':
    main()



import argparse
import logging

from datastation.batch_processing import batch_process
from datastation.config import init
from datastation.ds_pidsfile import load_pids
from datastation.dv_api import get_dataset_metadata, destroy_dataset
from re import match


def description_object_matches(description_text_pattern):
    def matches(description):
        value = description['dsDescriptionValue']['value']
        return match(description_text_pattern, value)

    return matches


def has_directoryLabel_different_from(file_metadata, dir_label):
    return 'directoryLabel' not in file_metadata or file_metadata['directoryLabel'] != dir_label


def destroy_placeholder_dataset(server_url, api_token, pid, description_text_pattern, dry_run):
    blocker = False
    try:
        dataset_metadata = get_dataset_metadata(server_url, api_token, pid)
        dsDescription = next(
            filter(lambda m: m['typeName'] == 'dsDescription', dataset_metadata['metadataBlocks']['citation']['fields']))
        descriptions = dsDescription['value']

        if len(list(filter(description_object_matches(description_text_pattern), descriptions))) == 0:
            blocker = True
            logging.warning("No description found matching pattern '{}'".format(description_text_pattern))
        else:
            logging.debug("Description with text pattern found: OK")

        files = dataset_metadata['files']

        if len(files) > 4:
            blocker = True
            logging.warning("More than 4 files found: {}".format(files))
        else:
            logging.debug("Found {} files <= 4: OK".format(len(files)))

        non_easy_migration_files = list(filter(lambda m: has_directoryLabel_different_from(m, 'easy-migration'), files))

        if len(non_easy_migration_files) > 0:
            blocker = True
            logging.warning("Files other than 'easy-migration' found: {}".format(non_easy_migration_files))
        else:
            logging.debug("Only found easy-migration files: OK")
    except Exception as e:
        blocker = True
        logging.warning("Could not perform checks on dataset {}".format(pid), exc_info=e)


    if blocker:
        logging.warning("BLOCKERS FOUND, NOT PERFORMING DESTROY FOR {}".format(pid))
    elif dry_run:
        logging.info("--dry-run option specified. Not performing destroy for {}".format(pid))
    else:
        logging.info("Destroying {}".format(pid))
        destroy_dataset(server_url, api_token, pid)


def main():
    config = init()
    server_url = config['dataverse']['server_url']
    api_token = config['dataverse']['api_token']
    description_text_pattern = config['migration_placeholders']['description_text_pattern']

    parser = argparse.ArgumentParser(
        description='Destroy metadata-only placeholders for datasets that have not been migrated yet')
    pid_or_file = parser.add_mutually_exclusive_group()
    pid_or_file.add_argument('-p', '--pid', dest='pid', help='Pid of a single placeholder dataset to destroy')
    pid_or_file.add_argument('-d', '--datasets', dest='pid_file', help='The input file with the dataset pids')
    parser.add_argument('--dry-run', dest='dry_run', help="only logs the actions, nothing is executed",
                        action='store_true')

    args = parser.parse_args()

    if args.pid is not None:
        destroy_placeholder_dataset(server_url, api_token, args.pid, description_text_pattern, args.dry_run)
    if args.pid_file is not None:
        pids = load_pids(args.pid_file)
        batch_process(pids,
                      lambda pid: destroy_placeholder_dataset(server_url, api_token, pid, description_text_pattern,
                                                              args.dry_run))


if __name__ == '__main__':
    main()



import json
import os

# Start - XML related stuff, maybe not needed
from lxml import etree
import dicttoxml


# check if we can also write xml instead of json!
def get_xml_output(data):
    # should do dict to xml
    xml = dicttoxml.dicttoxml(data, custom_root='dataset')  # create_xml_tree(etree.Element("root"), data)
    xml_dom = etree.fromstring(xml)
    return etree.tostring(xml_dom, pretty_print=True)


def store_dataset_result_as_xml(pid, dataset_metadata, save_path):
    # construct filename from pid
    # pid = dataset_json['global_id']
    filename_base = pid.replace(':', '_')
    filename_base = filename_base.replace('/', '_')
    filename_ext = 'xml'
    filename = filename_base + '.' + filename_ext
    full_name = os.path.join(save_path, filename)
    with open(full_name, "wb") as outfile:
        outfile.write(get_xml_output(dataset_metadata))
    return


# End - XML related stuff

def construct_filename_base_from_pid(pid):
    filename_base = pid.replace(':', '_')
    filename_base = filename_base.replace('/', '_')
    return filename_base


def get_json_output(data):
    return json.dumps(data, indent=2)


# note that search results don't contain all metadata, especially it is lacking the custom blocks!
def store_dataset_result(pid, dataset_metadata, save_path):
    # construct filename from pid
    # pid = dataset_json['global_id']
    filename_base = construct_filename_base_from_pid(pid)
    filename_ext = 'json'
    filename = filename_base + '.' + filename_ext
    full_name = os.path.join(save_path, filename)
    with open(full_name, "w") as outfile:
        outfile.write(get_json_output(dataset_metadata))
    return



import argparse

from datastation.batch_processing import batch_process
from datastation.config import init
from datastation.ds_pidsfile import load_pids
from datastation.dv_api import publish_dataset


def publish_dataset_command(server_url, api_token, delay, pids_file, version_upgrade_type):
    pids = load_pids(pids_file)

    # Long delay because publish is doing a lot after the async. request is returning
    # and sometimes datasets get locked
    batch_process(pids, lambda pid: publish_dataset(server_url, api_token, pid, version_upgrade_type), delay)


def main():
    config = init()
    parser = argparse.ArgumentParser(description='Publishes datasets with the pids in the given input file')
    parser.add_argument('-d', '--datasets', dest='pids_file', help='The input file with the dataset pids')
    parser.add_argument('-p', '--pid', help="Pid of the dataset to be published.")
    parser.add_argument('-t', '--type', dest='version_upgrade_type', default='major',
                        help='The type of version upgrade, "minor" or "updatecurrent" (only for superusers) for '
                             'metadata changes, default is "major", which is also needed for the initial version.')
    parser.add_argument('--delay', default=5.0, help="Delay in seconds")
    args = parser.parse_args()

    server_url = config['dataverse']['server_url']
    api_token = config['dataverse']['api_token']

    if args.pid is not None:
        publish_dataset(server_url, api_token, args.delay, args.pid, args.version_upgrade_type)
    else:
        publish_dataset_command(server_url, api_token, args.pids_file, args.version_upgrade_type)


if __name__ == '__main__':
    main()




# for iteration example see: https://guides.dataverse.org/en/5.6/api/search.html#iteration
# in a lot of cases we just want the pids and handle further processing elsewhere
# would be nice if we could make the 'paging' logic into a generic solution 'process_search'?
from datastation.dv_api import search


def get_dataset_pids_from_search(server_url, subtree):
    """
    Retrieve dataset pids from search results
    :param server_url:
    :param subtree:
    :return:
    """
    pids = []
    # pagination
    rows = 100
    start = 0
    page = 1
    condition = True  # emulate do-while
    while condition:
        result = search(server_url, subtree, start, rows)
        total_count = result['total_count']
        print("=== Page", page, "===")
        print("start:", start, " total:", total_count)
        for i in result['items']:
            # process/filter the result items, the dataset
            print("- ", i['global_id'], "(" + i['name'] + ")")
            # note that we only want the pids, it's not optimal with the use of resources
            # would be nice if the search API allowed specifying that we only want the pids (GraphQL like)
            pids.append(i['global_id'])
        start = start + rows
        page += 1
        condition = start < total_count
    return pids




# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['datastation', 'datastation.scripts']

package_data = \
{'': ['*']}

install_requires = \
['dicttoxml>=1.7.4,<2.0.0',
 'lxml>=4.8.0,<5.0.0',
 'psycopg>=3.0.16,<4.0.0',
 'pyYAML>=6.0,<7.0',
 'requests>=2.26.0,<3.0.0']

entry_points = \
{'console_scripts': ['dans-bag-validate = '
                     'datastation.scripts.validate_dans_bag:main',
                     'dv-datacite-records-update = '
                     'datastation.scripts.update_datacite_records:main',
                     'dv-dataset-add-role-assignment = '
                     'datastation.scripts.add_role_assignments:main',
                     'dv-dataset-delete-draft = '
                     'datastation.scripts.delete_draft_datasets:main',
                     'dv-dataset-delete-role-assignment = '
                     'datastation.scripts.delete_role_assignments:main',
                     'dv-dataset-destroy-migration-placeholder = '
                     'datastation.scripts.dataset_destroy_migration_placeholder:main',
                     'dv-dataset-find-with-role-assignment = '
                     'datastation.scripts.find_datasets_with_roleassignment:main',
                     'dv-dataset-oai-harvest = '
                     'datastation.scripts.oai_harvest:main',
                     'dv-dataset-publish = '
                     'datastation.scripts.publish_datasets:main',
                     'dv-dataset-reindex = '
                     'datastation.scripts.reindex_datasets:main',
                     'dv-dataset-replace-metadata-field-values = '
                     'datastation.scripts.replace_metadata_field_values:main',
                     'dv-dataset-retrieve-metadata = '
                     'datastation.scripts.retrieve_dataset_metadata:main',
                     'dv-dataset-retrieve-metadata-field = '
                     'datastation.scripts.retrieve_dataset_metadata_field:main',
                     'dv-dataset-unlock = '
                     'datastation.scripts.unlock_datasets:main',
                     'dv-dataverse-oai-harvest = '
                     'datastation.scripts.oai_harvest:main',
                     'dv-dataverse-retrieve-pids = '
                     'datastation.scripts.retrieve_dataset_pids:main',
                     'dv-file-prestage = '
                     'datastation.scripts.prestage_files:main',
                     'dv-user-import = datastation.scripts.import_user:main',
                     'ingest-flow-copy-batch-to-ingest-area = '
                     'datastation.scripts.ingest_flow_copy_batch_to_ingest_area:main',
                     'ingest-flow-move-batch-to-ingest-area = '
                     'datastation.scripts.ingest_flow_move_batch_to_ingest_area:main',
                     'ingest-flow-progress-report = '
                     'datastation.scripts.ingest_flow_progress_report:main',
                     'ingest-flow-start-import = '
                     'datastation.scripts.ingest_flow_start_import:main',
                     'ingest-flow-start-migration = '
                     'datastation.scripts.ingest_flow_start_migration:main']}

setup_kwargs = {
    'name': 'dans-datastation-tools',
    'version': '0.9.0',
    'description': 'Command line utilities for Data Station application management',
    'long_description': None,
    'author': 'DANS-KNAW',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import argparse
import json
import logging
import os
import csv
from email import encoders
from email.mime.application import MIMEApplication

import requests
import typing
import yaml

from datastation.config import init
from email.mime.multipart import MIMEMultipart


def validate_dans_bag(path, package_type, level, validator_url, accept_json, is_dry_run, output_writer, is_first=True):
    command = {
        'bagLocation': os.path.abspath(path),
        'packageType': package_type,
        'level': level
    }

    msg = MIMEMultipart("form-data")
    p = MIMEApplication(json.dumps(command), "json", _encoder=encoders.encode_noop)
    p.add_header("Content-Disposition", "form-data; name=command")
    msg.attach(p)

    body = msg.as_string().split('\n\n', 1)[1]
    headers = dict(msg.items())
    headers.update({'Accept': 'application/json' if accept_json else 'text/plain'})

    if is_dry_run:
        logging.info("Only printing command, not sending it...")
        print(msg.as_string())
    else:
        r = requests.post('{}/validate'.format(validator_url), data=body,
                          headers=headers)
        print(r.text)
        if accept_json:
            result = json.loads(r.text)
        else:
            result = yaml.safe_load(r.text)

        output_writer(result, is_first)

    return 0


def validate_dans_bag_in_deposit(path, package_type, level, validator_url, accept_json, is_dry_run, output_writer,
                                 is_first=True):
    subdirs = get_subdirs(path)
    if len(subdirs) == 1:
        validate_dans_bag(subdirs[0], package_type, level, validator_url, accept_json, is_dry_run, output_writer,
                          is_first)
    else:
        print("ERROR: deposit found with {} subdirectories. There should be exactly one".format(len(subdirs)))


def get_subdirs(dir):
    return list(filter(lambda d: os.path.isdir(d), map(lambda d: os.path.join(dir, d), os.listdir(dir))))


def validate_command(path, package_type, level, service_baseurl, accept_json, dry_run, output_writer):
    if os.path.exists("{}/bagit.txt".format(path)):
        logging.info("Found one bag at {}".format(path))
        validate_dans_bag(path, package_type, level, service_baseurl, accept_json, dry_run, output_writer)
    elif os.path.exists("{}/deposit.properties".format(path)):
        logging.info("Found a deposit at {}".format(path))
        validate_dans_bag_in_deposit(path, package_type, level, service_baseurl, accept_json, dry_run, output_writer)
    else:
        logging.info("Not a bag or a deposit, assuming batch of deposits")
        subdirs = get_subdirs(path)
        logging.info("Found {} deposits to validate".format(len(subdirs)))
        is_first = True
        for d in subdirs:
            logging.debug("Validating {}".format(d))
            validate_dans_bag_in_deposit(os.path.join(path, d), package_type, level, service_baseurl, accept_json,
                                         dry_run, output_writer, is_first)
            is_first = False


def create_csv_result_writer(csv_writer: csv.DictWriter):
    def result_writer(result, is_first: bool):
        csv_writer.writerow({
            'DEPOSIT': result["Bag location"],
            'BAG': result["Name"],
            'COMPLIANT': result["Is compliant"],
            'RULE VIOLATIONS': result["Rule violations"]
        })

    return result_writer


def create_json_result_writer(f: typing.TextIO):
    def result_writer(result, is_first: bool):
        if not is_first:
            f.write(", ")
        f.write(json.dumps(result))

    return result_writer


def create_yaml_result_writer(f: typing.TextIO):
    def result_writer(result, is_first: bool):
        f.write(yaml.dump([result]))

    return result_writer


def main():
    config = init()
    parser = argparse.ArgumentParser(
        description='Validate one or more bags to see if they comply with the DANS BagIt Profile v1')
    parser.add_argument('path', metavar='<batch-or-deposit-or-bag>',
                        help='Directory containing a bag, a deposit or a batch of deposits')
    parser.add_argument('-m', '--migration', dest='is_migration', action='store_true',
                        help='Validate the bag as a migration')
    parser.add_argument('--with-data-station-context', dest='in_context', action='store_true',
                        help='Validate in Data Station context')
    parser.add_argument('-d', '--dry-run', dest='dry_run', action='store_true',
                        help='Only print command to be sent to server, but do not actually send it')
    parser.add_argument('-o', '--out-file', dest='out_file',
                        required=True,
                        help='Output file to save results to. If the file name ends in .csv the output is saved as CSV, '
                             'if .json as JSON and otherwise as Yaml')

    args = parser.parse_args()
    service_baseurl = config['dans_bag_validator']['service_baseurl']
    package_type = "MIGRATION" if args.is_migration else "DEPOSIT"
    level = "WITH-DATA-STATION-CONTEXT" if args.in_context else "STAND-ALONE"
    dry_run = args.dry_run
    path = args.path

    with(open(args.out_file, 'w')) as f:
        if args.out_file.endswith('.csv'):
            fieldnames = ['DEPOSIT', 'BAG', 'COMPLIANT', 'RULE VIOLATIONS']
            csv_writer = csv.DictWriter(f, fieldnames=fieldnames)
            csv_writer.writeheader()
            validate_command(path, package_type, level, service_baseurl, accept_json=True, dry_run=dry_run,
                             output_writer=create_csv_result_writer(csv_writer))
        else:
            accept_json = args.out_file.endswith('.json')
            try:
                if accept_json:
                    f.write("[")
                validate_command(path, package_type, level, service_baseurl, accept_json, dry_run,
                                 create_json_result_writer(f) if accept_json else create_yaml_result_writer(f))
            finally:
                if accept_json:
                    f.write("]")


if __name__ == '__main__':
    main()



import time
import logging


def batch_process(pids, process_action_func, delay=0.1, fail_on_first_error=True):

    num_pids = len(pids)
    logging.info("Start batch processing on {} datasets".format(num_pids))
    num = 0
    for pid in pids:
        num += 1
        logging.info("[{} of {}] Processing dataset with pid: {}".format(num, num_pids, pid))
        try:
            process_action_func(pid)
        except Exception as e:
            logging.exception("Exception occurred", exc_info=True)
            if fail_on_first_error:
                logging.error("Stop processing because of an exception:  {}".format(e))
                break
            logging.debug("fail_on_first_error is False, continuing...")
        if delay > 0 and num < num_pids:
            logging.debug("Sleeping for {} seconds...".format(delay))
            time.sleep(delay)
    logging.info("Done processing {} out of {} datasets".format(num, num_pids))
